using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class BaseGame
    {        
        //Make the Input objects available to all classes that inherit
        //from base game (they will be the same objects since it's
        //declared as static)
        static protected InputManager mInputManager = new InputManager();
        public InputManager Input
        {
            get { return mInputManager; }
        }

        //Make the Font objects available to all classes that inherit
        //from base game (they will be the same objects since it's
        //declared as static)
        static protected FontManager mFontManager = new FontManager();
        public FontManager Font
        {
            get { return mFontManager; }
        }

        //Make the Texture objects available to all classes that inherit
        //from base game (they will be the same objects since it's
        //declared as static)
        static protected TextureManager mTextureManager = new TextureManager();
        public TextureManager Texture
        {
            get { return mTextureManager; }
        }

        //Make a SpriteBatch object avialable to all classes
        static protected SpriteBatch mBatch;

        //Store the Display heights and widths and make them available to all classes
        static protected int mDisplayWidth;
        static protected int mDisplayHeight;

        //Create a new Random object to be used by all classes
        static protected Random mRandom = new Random();

        //Store references to the Graphics and Content objects and make them available
        //to all classes
        static protected GraphicsDevice mGraphics;
        static protected ContentManager mContent;

        //The timespan to compare to when time isn't set
        static protected TimeSpan mTimeNotSet = new TimeSpan(-9, -9, -9);

        public BaseGame()
        {            
        }

        //Load all the appropriate content in the base management classes and store the appropriate references
        public void LoadContent(GraphicsDevice theGraphics, ContentManager theContent)
        {           
            mGraphics = theGraphics;
            mContent = theContent;

            mDisplayWidth = theGraphics.Viewport.Width;
            mDisplayHeight = theGraphics.Viewport.Height;

            mFontManager.LoadContent(mContent, mDisplayWidth, mDisplayHeight);
            mTextureManager.LoadContent(mContent);

            mBatch = new SpriteBatch(theGraphics);            
        }

    }
}
