using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class FontManager
    {
        //Define the paths for all of th font objects
        const string Font_Instructions = "Fonts/Instruction";

        //Define the font objects
        SpriteFont mInstructionText;
     
        //All of the available fonts
        public enum Font
        {
            Instructions
        }

        //Alignment options
        public enum Alignment
        {
            CenterScreen,
            CenterHorizontally,
            CenterVertically,
            None
        }

        //Store the current display height and width
        int mDisplayWidth;
        int mDisplayHeight;

        public FontManager()
        {
        }

        //Load all of the font objects and store the current display height and width
        public void LoadContent(ContentManager theContent, int theDisplayWidth, int theDisplayHeight)
        {
            mInstructionText = theContent.Load<SpriteFont>(Font_Instructions);
            mDisplayWidth = theDisplayWidth;
            mDisplayHeight = theDisplayHeight;
        }

        //Draw text to the screen with the selected font and alignment
        public void DrawText(SpriteBatch theBatch, Alignment theAlignment, Vector2 thePosition, string theText, Font theFont)
        {          
            switch (theAlignment)
            {
                case Alignment.None:
                    {
                        theBatch.DrawString(SelectFont(theFont), theText, thePosition, Color.White);     
                        break;
                    }

                case Alignment.CenterScreen:
                    {
                        SpriteFont aFont = SelectFont(theFont);
                        int aPositionX = (int)((mDisplayWidth / 2) - (aFont.MeasureString(theText).X / 2));
                        int aPositionY = (int)((mDisplayHeight / 2) - (aFont.LineSpacing / 2));

                        theBatch.DrawString(SelectFont(theFont), theText, new Vector2(aPositionX, aPositionY), Color.White);

                        break;
                    }

                case Alignment.CenterHorizontally:
                    {
                        SpriteFont aFont = SelectFont(theFont);
                        int aPositionX = (int)((mDisplayWidth / 2) - (aFont.MeasureString(theText).X / 2));
                        int aPositionY = (int)(thePosition.Y);

                        theBatch.DrawString(SelectFont(theFont), theText, new Vector2(aPositionX, aPositionY), Color.White);

                        break;
                    }

                case Alignment.CenterVertically:
                    {
                        SpriteFont aFont = SelectFont(theFont);
                        int aPositionX = (int)(thePosition.X);
                        int aPositionY = (int)((mDisplayHeight / 2) - (aFont.LineSpacing / 2));

                        theBatch.DrawString(SelectFont(theFont), theText, new Vector2(aPositionX, aPositionY), Color.White);
                        
                        break;
                    }
            }
        }

        //Return a font object based on the the font chosen
        public SpriteFont SelectFont(Font theFont)
        {
            switch (theFont)
            {
                case Font.Instructions:
                    {
                        return mInstructionText;
                    }
            }

            return mInstructionText;
        }
    }
}
