using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class InputManager
    {
        //Define the current input state objects
        KeyboardState mCurrentKeyboardState;
        GamePadState mCurrentGamePadState;

        //Define the previous input state objects
        KeyboardState mPreviousKeyboardState;
        GamePadState mPreviousGamePadState;

        //Define the available input buttons for the game
        public enum Input
        {
            Start,
            Exit,
            Pause,
            A
        }

        public InputManager()
        {
        }

        //Called when the main game loop enters the Update method. Get the current state of
        //the keyboard and the gamepad
        public void UpdateBegin()
        {
           mCurrentKeyboardState = Keyboard.GetState();
           mCurrentGamePadState = GamePad.GetState(PlayerIndex.One);
        }

        //Called when the main game loop exits the Update method. Stores the state of
        //the keyboard and the gamepad in the previous state objects
        public void UpdateEnd()
        {
            mPreviousKeyboardState = mCurrentKeyboardState;
            mPreviousGamePadState = mCurrentGamePadState;
        }

        //Check to see if a particular input has been pressed
        public bool Pressed(Input theInput)
        {        
            switch (theInput)
            {
                case Input.Start:
                    {
                        return IsStartPressed();
                    }

                case Input.Exit:
                    {
                        return IsExitPressed();
                    }

                case Input.A:
                    {
                        return IsAPressed();
                    }

                case Input.Pause:
                    {
                        return IsPausePressed();
                    }
            }

            return false;
        }
        
        private bool IsStartPressed()
        {
            if ((mCurrentKeyboardState.IsKeyDown(Keys.Enter) == true && mPreviousKeyboardState.IsKeyDown(Keys.Enter) == false)
                || (mCurrentGamePadState.Buttons.Start == ButtonState.Pressed && mPreviousGamePadState.Buttons.Start == ButtonState.Released))
            {
                return true;
            }

            return false;
        }

        private bool IsExitPressed()
        {
            if ((mCurrentKeyboardState.IsKeyDown(Keys.Escape) == true && mPreviousKeyboardState.IsKeyDown(Keys.Escape) == false)
                || (mCurrentGamePadState.Buttons.Back == ButtonState.Pressed && mPreviousGamePadState.Buttons.Back == ButtonState.Released))
            {
                return true;
            }

            return false;
        }

        private bool IsAPressed()
        {
            if ((mCurrentKeyboardState.IsKeyDown(Keys.Space) == true && mPreviousKeyboardState.IsKeyDown(Keys.Space) == false)
                || (mCurrentGamePadState.Buttons.A == ButtonState.Pressed && mPreviousGamePadState.Buttons.A == ButtonState.Released))
            {
                return true;
            }

            return false;
        }

        private bool IsPausePressed()
        {
            if ((mCurrentKeyboardState.IsKeyDown(Keys.P) == true && mPreviousKeyboardState.IsKeyDown(Keys.P) == false)
                || (mCurrentGamePadState.Buttons.Start == ButtonState.Pressed && mPreviousGamePadState.Buttons.Start == ButtonState.Released))
            {
                return true;
            }

            return false;
        }

    }
}
