using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class Car : Sprite
    {
        const double Speed_Default = .4;

        float mTireRotation = 0f;
            
        RenderTarget2D aCar;
        SpriteBatch mNewBatch;
        protected double mScale;

        protected Rectangle aCarRectangle;
        protected Rectangle aTireOneRectangle;
        protected Rectangle aTireTwoRectangle;

        protected Rectangle mCarRectangle;        
        public Rectangle CarPosition
        {
            get { return mCarRectangle; }
        }

        public override Rectangle CollisionRectangle
        {
            get 
            {
                mCollisionRectangle = mCarRectangle;
                mCollisionRectangle.Width -= 130;
                mCollisionRectangle.X -= 75;
                mCollisionRectangle.Y -= 30;
                mCollisionRectangle.Height -= 63;

                if (mCarRotation == Rotate.Down45Degrees)
                {
                    mCollisionRectangle.Y -= 20;
                    mCollisionRectangle.X += 35;
                    mCollisionRectangle.Width -= 10;
                    mCollisionRectangle.Height -= 10;
                }

                if (mCarRotation == Rotate.Up45Degrees)
                {
                    mCollisionRectangle.Y += 45;
                    mCollisionRectangle.X += 0;
                    mCollisionRectangle.Width -= 10;
                    mCollisionRectangle.Height -= 10;
                }

                return mCollisionRectangle; 
            }
        }

        public int PositionY
        {
            get { return mCarRectangle.Y; }
            set { mCarRectangle.Y = value; }
        }

        public int PositionX
        {
            get { return mCarRectangle.X; }
            set { mCarRectangle.X = value; }
        }

        public enum Rotate
        {
            Up45Degrees,
            Down45Degrees,
            None
        }

        Rotate mCarRotation = Rotate.None;
        public Rotate Rotation
        {
            get { return mCarRotation; }
            set { mCarRotation = value; }
        }

        Rectangle mSourceRectangle;
        protected Rectangle SourceRectangle
        {
            get { return mSourceRectangle; }
            set { mSourceRectangle = value; }
        }

        SpriteEffects mSpriteEffects;
        protected SpriteEffects Effects
        {
            get { return mSpriteEffects; }
            set { mSpriteEffects = value; }
        }

        public enum CarType
        {
            Cop,
            Robber
        }
        protected CarType mCarType = CarType.Cop;
        public CarType Type
        {
            get { return mCarType; }
        }

        private bool mVisible = true;
        public bool Visible
        {
            get { return mVisible; }
            set { mVisible = value; }
        }

        private double mSpeed = Speed_Default;
        public double Speed
        {
            get { return mSpeed; }
            set { mSpeed = value; }
        }

        Vector2 mTargetPosition;

        public Car(double theScale) : base(new Vector2(0,0), (int)(363 * theScale), (int)(210 * theScale), Color.White)
        {
            mScale = theScale;
            aCar = new RenderTarget2D(mGraphics, 363, 230, false, SurfaceFormat.Color, DepthFormat.Depth24);
            
            mNewBatch = new SpriteBatch(mGraphics);
        }

        public virtual void Update(GameTime theGameTime)
        {
            mTireRotation += .2f;
            if (mTireRotation > 1)
            {
                mTireRotation = 0f;
            }

            if (mCarRectangle.X > mDisplayWidth + mCarRectangle.Width)
            {
                mCarRectangle.X =  (int)(-5 * mScale) ;
            }

            
            switch (mCarRotation)
            {
                case Rotate.None:
                    {
                        MoveCar(mSpeed, 0, theGameTime);
                        break;
                    }

                case Rotate.Down45Degrees:
                    {
                        //MoveCar(3, 3);
                        MoveOnAngle(theGameTime);
                        break;
                    }

                case Rotate.Up45Degrees:
                    {
                        //MoveCar(3, -3);
                        MoveOnAngle(theGameTime);
                        break;
                    }
            }
        }

        protected virtual void UpdateCar()
        {
            aCarRectangle = new Rectangle(0, 35, (int)(363 * mScale), (int)(200 * mScale)); //200 high, 363 wided
            aTireOneRectangle = new Rectangle(aCarRectangle.X + (int)(135 * mScale), aCarRectangle.Y + (int)(152 * mScale), (int)(72 * mScale), (int)(72 * mScale));
            aTireTwoRectangle = new Rectangle(aCarRectangle.X + (int)(255 * mScale), aCarRectangle.Y + (int)(152 * mScale), (int)(72 * mScale), (int)(72 * mScale));
        }

        protected virtual void MoveCar(double theMovementX, double theMovementY, GameTime theGameTime)
        {
            theMovementX *= theGameTime.ElapsedGameTime.Milliseconds;
            theMovementY *= theGameTime.ElapsedGameTime.Milliseconds;

            mCarRectangle = new Rectangle(mCarRectangle.X + (int)theMovementX, mCarRectangle.Y + (int)theMovementY, (int)(363 * mScale), (int)(210 * mScale));
        }

        private void MoveOnAngle(GameTime theGameTime)
        {
            double aRotation = MathHelper.ToDegrees(45);

            switch (mCarRotation)
            {
                case Rotate.Up45Degrees:
                    {
                        double aSpeed = (mSpeed - .2);
                        if (aSpeed < .1)
                        {
                            aSpeed = .1;
                        }

                        aSpeed *= theGameTime.ElapsedGameTime.Milliseconds;

                        aRotation = MathHelper.ToRadians(-45);
                        mCarRectangle.X += (int)(Math.Cos(aRotation) * aSpeed);
                        mCarRectangle.Y += (int)(Math.Sin(aRotation) * aSpeed);
                        break;
                    }
                case Rotate.Down45Degrees:
                    {
                        double aSpeed = (mSpeed + .2) * theGameTime.ElapsedGameTime.Milliseconds;

                        aRotation = MathHelper.ToRadians(45);
                        mCarRectangle.X += (int)(Math.Cos(aRotation) * aSpeed);
                        mCarRectangle.Y += (int)(Math.Sin(aRotation) * aSpeed);
                        break;
                    }
            }



            if (Math.Abs(mCarRectangle.Y - mTargetPosition.Y) < (mSpeed * 10))
            {
                if (mCarRotation == Rotate.Up45Degrees)
                {
                    mCarRectangle.X += 50;
                }
                else
                {
                    mCarRectangle.X += 20;
                }

                mCarRotation = Rotate.None;
            }
        }        

        public void AngleCar(Vector2 theStartingPosition, Vector2 theTargetPosition, Rotate theAngle)
        {
            mCarRotation = theAngle;
            mTargetPosition = theTargetPosition;
            mCarRectangle.X = (int)theStartingPosition.X;
            mCarRectangle.Y = (int)theStartingPosition.Y;
        }

        public virtual void Draw(GameTime theGameTime)
        {
            if (Visible == false)
            {
                return;
            }

            float aRotation = 0;
            switch (mCarRotation)
            {
                case Rotate.Up45Degrees:
                    {
                        aRotation = MathHelper.ToRadians(-45);
                        break;
                    }
                case Rotate.Down45Degrees:
                    {
                        aRotation = MathHelper.ToRadians(45);
                        break;
                    }
            }

            mBatch.Draw(CarTexture(theGameTime), mCarRectangle, new Rectangle(0, 0, 363, 230), Color.White, aRotation, new Vector2((float)(363 * mScale), (float)(210 * mScale)), SpriteEffects.None, 0);

            base.Draw();
        }

        public virtual void DrawExtraCarPieces(Rectangle theCarRectangle, SpriteBatch theBatch, GameTime theGameTime, Texture2D theCarTexture)
        {
        }

        private Texture2D CarTexture(GameTime theGameTime)
        {            
            mGraphics.SetRenderTarget(aCar);
            mGraphics.Clear(ClearOptions.Target, Color.Transparent, 0, 0);
            
            mNewBatch.Begin();

            UpdateCar();      
            
            mNewBatch.Draw(Texture.Car, aCarRectangle, SourceRectangle, Color.White, 0, new Vector2(0, 0), Effects, 0);
            mNewBatch.Draw(Texture.Car, aTireOneRectangle, new Rectangle(68, 415, 72, 72), Color.White, mTireRotation, new Vector2(36, 36), SpriteEffects.None, 0);
            mNewBatch.Draw(Texture.Car, aTireTwoRectangle, new Rectangle(68, 415, 72, 72), Color.White, mTireRotation, new Vector2(36, 36), SpriteEffects.None, 0);

            DrawExtraCarPieces(aCarRectangle, mNewBatch, theGameTime, Texture.Car);

            mNewBatch.End();

            mGraphics.SetRenderTarget(null);
            mGraphics.Clear(Color.SlateGray);

            return aCar;
        }

        public void Reset()
        {
            mSpeed = Speed_Default;            
        }

    }
}
