using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace CopsAndRobbers
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        BaseGame mBaseGame = new BaseGame();

        TitleScreen mTitle;
        LevelScreen mLevel;

        enum Screen
        {
            Title,
            Game
        }
        Screen mCurrentScreen = Screen.Title;

        RenderTarget2D mZuneScreen;
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferHeight = 600;
            graphics.PreferredBackBufferWidth = 800;
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Load all of the game contents (audio, images, fonts)
            mBaseGame.LoadContent(this.graphics.GraphicsDevice, Content);

            PresentationParameters pp = graphics.GraphicsDevice.PresentationParameters;
            mZuneScreen = new RenderTarget2D(graphics.GraphicsDevice,
                                               pp.BackBufferWidth,
                                               pp.BackBufferHeight,
                                               false,
                                               pp.BackBufferFormat, DepthFormat.Depth24);

            graphics.GraphicsDevice.Clear(Color.Transparent);
            //graphics.GraphicsDevice.SetRenderTarget(mZuneScreen);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            //Begin the input Update to store the current state of the input objects
            mBaseGame.Input.UpdateBegin();

            //Check to see if the game should be exited
            if (mBaseGame.Input.Pressed(InputManager.Input.Exit) == true)
            {
                this.Exit();
            }

            switch (mCurrentScreen)
            {
                case Screen.Title:
                    {
                        //Create a new title screen object if
                        //one hasn't already been created
                        if (mTitle == null)
                        {
                            mTitle = new TitleScreen();
                        }

                        //Update the Title screen objects
                        mTitle.Update(gameTime);

                        //Check to see if it's time to start
                        //the game
                        if (mTitle.StartGame == true)
                        {
                            mTitle = null;
                            mCurrentScreen = Screen.Game;
                            mLevel = new LevelScreen();
                        }
                        break;
                    }

                case Screen.Game:
                    {
                        //Create a new Level object if one
                        //hasn't already been created
                        if (mLevel == null)
                        {
                            mLevel = new LevelScreen();
                        }

                        //Update the Level screen objects
                        mLevel.Update(gameTime);
                        break;
                    }
            }

            //Update the input object to store the current
            //states into the previous states
            mBaseGame.Input.UpdateEnd();
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.SlateGray);

            switch (mCurrentScreen)
            {
                case Screen.Title:
                    {
                        //Draw the Title screen objects
                        mTitle.Draw(gameTime);
                        break;
                    }

                case Screen.Game:
                    {
                        //Draw the Level screen objects
                        mLevel.Draw(gameTime);
                        break;
                    }
            }

            //graphics.GraphicsDevice.ResolveBackBuffer(mZuneScreen);
            //graphics.GraphicsDevice.Clear(Color.SlateGray);
            //////Now we redraw the scene after the ResolveBackBuffer call
            //spriteBatch.Begin(SpriteBlendMode.None,
            //                  SpriteSortMode.Immediate,
            //                  SaveStateMode.None);
            //spriteBatch.Draw(mZuneScreen, new Rectangle(0, 0, 400, 240), new Rectangle(0, 0, mZuneScreen.Width, mZuneScreen.Height), Color.White, 0.0f, Vector2.Zero, SpriteEffects.None, 0);
            //spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
