using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class LevelLayout : BaseGame
    {
        //The floors for the current level layout
        List<Floor> mFloors = new List<Floor>();
        public List<Floor> Floors
        {
            get { return mFloors; }
        }

        //The slopes for the current level layout
        List<Slope> mSlopes = new List<Slope>();
        public List<Slope> Slopes
        {
            get { return mSlopes; }
        }

        //The boxes for the current level layout
        List<Box> mBoxes = new List<Box>();
        public List<Box> Boxes
        {
            get { return mBoxes; }
        }

        //All of the available levels for loading
        public enum Levels
        {
            One
        }
                
        public LevelLayout()
        {
            LoadLevelOne();
        }

        //Draw the currently loaded level objects
        public void Draw()
        {        
            //Draw all of the boxes on the level
            foreach (Box aBox in mBoxes)
            {
                aBox.Draw();
            }

            //Draw all of the floors
            foreach (Floor aFloor in mFloors)
            {
                aFloor.Draw();
            }

            //Draw all of the slopes (and their attached ramps)
            foreach (Slope aSlope in mSlopes)
            {
                aSlope.Draw();
            }
       }
        
        //Load the level indicated
        public void LoadLevel(Levels theLevel)
        {
            switch (theLevel)
            {
                case Levels.One:
                    {
                        LoadLevelOne();
                        break;
                    }
            }
        }

        //Loads the layout for the first level. 
        private void LoadLevelOne()
        {
            int aSpacing = (int)(mDisplayHeight / 3) + 15;

            //Add the first story floor
            Floor aFloor = new Floor(new Vector2(0, aSpacing - 40), mDisplayWidth, Color.Gray);
            mFloors.Add(aFloor);

            //Add the second story floor
            aFloor = new Floor(new Vector2(0, (aSpacing - 40) * 2), mDisplayWidth, Color.Gray);
            mFloors.Add(aFloor);

            //Add the third story floor
            aFloor = new Floor(new Vector2(0, (aSpacing - 40) * 3), mDisplayWidth, Color.Gray);
            mFloors.Add(aFloor);

            //Adds the first down slant
            Slope aSlope = new Slope(Slope.Slant.Right, new Vector2(150, aSpacing - 25), aSpacing + 25, Color.Gray);
            mSlopes.Add(aSlope);

            //Add the second down slant
            aSlope = new Slope(Slope.Slant.Right, new Vector2(mDisplayWidth - 250, (aSpacing * 2) - 65), aSpacing + 25, Color.Gray);
            mSlopes.Add(aSlope);

            //Add the first up slant
            aSlope = new Slope(Slope.Slant.Left, new Vector2(mDisplayWidth - 150, aSpacing - 25), aSpacing + 25, Color.Gray);
            mSlopes.Add(aSlope);

            //Add the second up slant
            aSlope = new Slope(Slope.Slant.Left, new Vector2(250, (aSpacing * 2) - 65), aSpacing + 25, Color.Gray);
            mSlopes.Add(aSlope);

            //Add a black box at the bottom of the level
            Box aBox = new Box(new Vector2(0, (aSpacing * 3) - 100), mDisplayWidth, 100, Color.Black);
            mBoxes.Add(aBox);
        }

    }
}
