using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class Slope : Floor
    {
        //Define the slants for the Slope
        public enum Slant
        {
            Right,
            Left
        }

        //Store the current rotatin for the Slope
        float mRotation;

        Rectangle mSource;
        Vector2 mOrigin;

        //The Ramps (on and off) associated with the Slope
        List<Ramp> mRamps = new List<Ramp>();
        public List<Ramp> Ramps
        {
            get { return mRamps; }
        }

        //Expose the type of Slant
        Slant mSlant;
        public Slant SlopeSlant
        {
            get { return mSlant; }
        }

        public Slope(Slant theSlant, Vector2 thePosition, int theWidth, Color theColor) : base(thePosition, theWidth, theColor)
        {
            mSlant = theSlant;

            if (theSlant == Slant.Right)
            {
                //Create the rotation 
                mRotation = MathHelper.ToRadians(45);

                //Position and add the on and off ramps for the Right slanted slope
                Ramp aRamp = new Ramp(new Vector2(thePosition.X - 15, thePosition.Y - 15));
                mRamps.Add(aRamp);

                aRamp = new Ramp(new Vector2(thePosition.X + theWidth - 100, thePosition.Y + theWidth - 80));
                mRamps.Add(aRamp);
            }
            else
            {
                //Create the rotation
                mRotation = MathHelper.ToRadians(135);

                //Position and add the on and off ramps for the Left slanted slope                
                Ramp aRamp = new Ramp(new Vector2(thePosition.X - 30, thePosition.Y - 15));
                mRamps.Add(aRamp);

                aRamp = new Ramp(new Vector2(thePosition.X - theWidth + 55, thePosition.Y + theWidth - 80));
                mRamps.Add(aRamp);
            }

            //Store the image source and origin for the slopes
            mSource = new Rectangle(0, 0, Texture.Wall.Width, Texture.Wall.Height);
            mOrigin = new Vector2(0, Floor_Height/2);
        }

        public override void Draw()
        {
            //Draw the slope
            mBatch.Draw(Texture.Wall, mDestination, mSource, mColor, mRotation, mOrigin, SpriteEffects.None, 0);

            //Draw the ramps associated with the Slope
            foreach (Ramp aRamp in mRamps)
            {
                aRamp.Draw();
            }
        }
    }
}
