using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class PowerUp : Sprite
    {

        const double PowerUp_Scale = .15;
        const int PowerUp_Width = 250;
        const int PowerUp_Height = 225;
        const double PowerUp_Blink = 90;

        const int Level1_PositionY = 138;
        const int Level2_PositionY = 312;
        const int Level3_PositionY = 485;

        int mPowerUpWidth;
        int mPowerUpHeight;

        Rectangle mFlipRectangle = new Rectangle(0, 0, 250, 225);
        Rectangle mSpeedUpRectangle = new Rectangle(256, 231, 250, 225);
        Rectangle mSlowDownRectangle = new Rectangle(0, 453, 250, 225);
        Rectangle mTimeBoostRectangle = new Rectangle(0, 225, 250, 225);
        Rectangle mPrisonBreakRectangle = new Rectangle(256, 0, 250, 225);
        
        //The available types of Powerups
        public enum PowerUpType
        {
            SpeedUp,
            SlowDown,
            TimeBoost,
            PrisonBreak,
        }

        //The type of Powerup this is
        PowerUpType mCurrentType = PowerUpType.SpeedUp;
        public PowerUpType CurrentType
        {
            get { return mCurrentType; }
        }

        //The position of the PowerUp
        Vector2 mPosition;
        public Vector2 Position
        {
            get { return mPosition; }
            set { mPosition = value; }
        }

        //The collision area for the PowerUp
        public override Rectangle CollisionRectangle
        {
            get 
            {
                mCollisionRectangle.X = (int)mPosition.X;
                mCollisionRectangle.Y = (int)mPosition.Y;
                mCollisionRectangle.Width = 37;
                mCollisionRectangle.Height = 32;

                return mCollisionRectangle;
            }
        }

        //Determines if the PowerUp is visible or not
        bool mVisible = true;
        public bool Visible
        {
            get { return mVisible; }
            set { mVisible = value; }
        }

        //Determines if the PowerUp is available for pickup
        bool mAvailable = false;
        public bool Available
        {
            get { return mAvailable; }
            set
            {
                //If the Power up becomes un-available, then the
                //power up should not be visible
                if (value == false)
                {
                    mVisible = false;
                }
                mAvailable = value;
            }
        }

        TimeSpan mBlinking = mTimeNotSet;
        double mBlink = PowerUp_Blink;

        public PowerUp(ContentManager theContent)
            : base(new Vector2(0, 0), (int)(PowerUp_Width * PowerUp_Scale), (int)(PowerUp_Height * PowerUp_Scale), Color.White)
        {
            mPowerUpWidth = (int)(PowerUp_Width * PowerUp_Scale);
            mPowerUpHeight = (int)(PowerUp_Height * PowerUp_Scale);
        }

        public void Update(GameTime theGameTime)
        {
            if (mBlinking != mTimeNotSet)
            {
                mBlink -= theGameTime.ElapsedGameTime.Milliseconds;
                if (mBlink < 0)
                {
                    mBlink = PowerUp_Blink;
                    mVisible = !mVisible;
                }

                mBlinking -= theGameTime.ElapsedGameTime;
                if (mBlinking <= new TimeSpan(0, 0, 0))
                {
                    mAvailable = true;
                    mVisible = true;
                    mBlinking = mTimeNotSet;
                }               
            }            
        }

        public override void Draw()
        {
            if (mVisible == true)
            {
                mBatch.Draw(Texture.PowerUp, new Rectangle((int)mPosition.X, (int)mPosition.Y, mPowerUpWidth, mPowerUpHeight), SourceRectangle(), Color.White);
            }

            base.Draw();
        }

        private Rectangle SourceRectangle()
        {
            Rectangle aSourceRectangle = mSpeedUpRectangle;
            switch (mCurrentType)
            {
                case PowerUpType.PrisonBreak:
                    {
                        aSourceRectangle = mPrisonBreakRectangle;
                        break;
                    }

                case PowerUpType.SlowDown:
                    {
                        aSourceRectangle = mSlowDownRectangle;
                        break;
                    }

                case PowerUpType.SpeedUp:
                    {
                        aSourceRectangle = mSpeedUpRectangle;
                        break;
                    }

                case PowerUpType.TimeBoost:
                    {
                        aSourceRectangle = mTimeBoostRectangle;
                        break;
                    }
            }
            return aSourceRectangle;
        }

        public void CreatePowerUp()
        {
            mBlinking = new TimeSpan(0, 0, 3);
            Available = false;           

            mCurrentType = (PowerUpType)mRandom.Next(0, 4);
            mPosition.X = mRandom.Next(50, mDisplayWidth - 50);
            
            switch ((int)mRandom.Next(0, 3))
            {
                case 0:
                    {
                        mPosition.Y = Level1_PositionY;
                        break;
                    }

                case 1:
                    {
                        mPosition.Y = Level2_PositionY;
                        break;
                    }

                case 2:
                    {
                        mPosition.Y = Level3_PositionY;
                        break;
                    }
            }
        }

    }
}
