using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class PowerUpManager : BaseGame
    {
        TimeSpan mNextPowerUpAppears;

        string mPowerUpText;
        public string PowerUpText
        {
            get { return mPowerUpText; }
            set { mPowerUpText = value; }
        }

        TimeSpan mPowerUpTextDisplay = mTimeNotSet;
        
        TimeSpan mCopPowerUp = new TimeSpan(0, 0, 3);
        TimeSpan mRobberPowerUp = new TimeSpan(0, 0, 3);

        public PowerUpManager()
        {
            mPowerUp = new PowerUp(mContent);
            mPowerUp.Available = false;
            mNextPowerUpAppears = new TimeSpan(0, 0, 15);
        }

        PowerUp mPowerUp;
        public PowerUp CurrentPowerup
        {
            get { return mPowerUp; }
        }

        public void Update(GameTime theGameTime, CopCar theCopCar, List<RobberCar> theRobbers, ref TimeSpan theGameTimeRemaining, ref int theRobbersCaught)
        {
            if (mPowerUpTextDisplay != new TimeSpan(-9, -9, -9))
            {
                mPowerUpTextDisplay -= theGameTime.ElapsedGameTime;
                if (mPowerUpTextDisplay < new TimeSpan(0, 0, 0))
                {
                    mPowerUpText = "";
                    mPowerUpTextDisplay = new TimeSpan(-9, -9, -9);
                }
            }

            mNextPowerUpAppears -= theGameTime.ElapsedGameTime;
            if (mNextPowerUpAppears <= new TimeSpan(0, 0, 0) && mPowerUp.Available == false)
            {
                CreatePowerUp();
            }            

            if (mCopPowerUp != new TimeSpan(-9, -9, -9))
            {
                mCopPowerUp -= theGameTime.ElapsedGameTime;
                if (mCopPowerUp <= new TimeSpan(0, 0, 0) && mCopPowerUp != new TimeSpan(-9, -9, -9))
                {
                    theCopCar.Reset();
                    mCopPowerUp = new TimeSpan(-9, -9, -9);
                }
            }

            if (mRobberPowerUp != new TimeSpan(-9, -9, -9))
            {
                mRobberPowerUp -= theGameTime.ElapsedGameTime;
                if (mRobberPowerUp <= new TimeSpan(0, 0, 0) && mRobberPowerUp != new TimeSpan(-9, -9, -9))
                {
                    foreach (RobberCar aRobber in theRobbers)
                    {
                        aRobber.Reset();
                    }
                    mRobberPowerUp = new TimeSpan(-9, -9, -9);
                }
            }

            CollisionCheck(theCopCar, theRobbers, ref theGameTimeRemaining, ref theRobbersCaught);
            foreach (RobberCar aRobber in theRobbers)
            {
                if (aRobber.Visible == true)
                {
                    CollisionCheck(aRobber, theRobbers, ref theGameTimeRemaining, ref theRobbersCaught);
                }
            }

            mPowerUp.Update(theGameTime);

        }

        public void Draw()
        {
            mPowerUp.Draw();
            if (mPowerUpTextDisplay != new TimeSpan(-9, -9, -9))
            {
                Font.DrawText(mBatch, FontManager.Alignment.CenterScreen, new Vector2(0, 0), mPowerUpText, FontManager.Font.Instructions);
            }
        }

        public void CollisionCheck(Car theCar, List<RobberCar> theRobbers, ref TimeSpan theGameTimeRemaining, ref int theRobbersCaught)
        {
            if (theCar.CollisionRectangle.Intersects(mPowerUp.CollisionRectangle) == true)
            {
                if (mPowerUp.Available == true)
                {
                    mPowerUp.Available = false;
                    if (theCar.Type == Car.CarType.Cop)
                    {
                        CopPowerUp(theCar, theRobbers, ref theGameTimeRemaining, ref theRobbersCaught);
                    }
                    else
                    {
                        RobberPowerUp(theRobbers, ref theGameTimeRemaining, ref theRobbersCaught);
                    }
                }
            }
        }

        private void CreatePowerUp()
        {
            mPowerUp.CreatePowerUp();
            mPowerUp.Available = false;
            mNextPowerUpAppears = new TimeSpan(0, 0, mRandom.Next(5, 11));
        }

        private void RobberPowerUp(List<RobberCar> theRobbers, ref TimeSpan theGameTimeRemaining, ref int theRobbersCaught)
        {
            switch (mPowerUp.CurrentType)
            {
                case PowerUp.PowerUpType.PrisonBreak:
                    {
                        PrisonBreak(theRobbers, ref theRobbersCaught);
                        break;
                    }

                case PowerUp.PowerUpType.SlowDown:
                    {
                        SlowDownRobbers(theRobbers);
                        break;
                    }

                case PowerUp.PowerUpType.SpeedUp:
                    {
                        SpeedUpRobbers(theRobbers);
                        break;
                    }

                case PowerUp.PowerUpType.TimeBoost:
                    {
                        DecreaseTime(ref theGameTimeRemaining);
                        break;
                    }
            }
        }

        private void CopPowerUp(Car theCar, List<RobberCar> theRobbers, ref TimeSpan theGameTimeRemaining, ref int theRobbersCaught)
        {
            switch (mPowerUp.CurrentType)
            {
                case PowerUp.PowerUpType.PrisonBreak:
                    {
                        PrisonBust(theRobbers, ref theRobbersCaught);
                        break;
                    }

                case PowerUp.PowerUpType.SlowDown:
                    {
                        SlowDownCop(theCar);
                        break;
                    }

                case PowerUp.PowerUpType.SpeedUp:
                    {
                        SpeedUpCop(theCar);
                        break;
                    }

                case PowerUp.PowerUpType.TimeBoost:
                    {
                        IncreaseTime(ref theGameTimeRemaining);
                        break;
                    }
            }
        }

        private void SpeedUpCop(Car theCar)
        {
            if (theCar.Speed == .4)
            {
                theCar.Speed += .3;
            }
            mCopPowerUp = new TimeSpan(0, 0, 3);

            mPowerUpText = "Speed Boost!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void SpeedUpRobbers(List<RobberCar> theRobbers)
        {
            //Catch one of the Robbers
            foreach (RobberCar aRobber in theRobbers)
            {
                if (aRobber.Visible == true)
                {
                    if (aRobber.Speed == .4)
                    {
                        aRobber.Speed += .3;
                    }
                }
            }

            mRobberPowerUp = new TimeSpan(0, 0, 3);

            mPowerUpText = "Too Fast For You!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void SlowDownCop(Car theCar)
        {
            if (theCar.Speed == .4)
            {
                theCar.Speed -= .3;
            }
            mCopPowerUp = new TimeSpan(0, 0, 3);

            mPowerUpText = "Super Slowdown...";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void SlowDownRobbers(List<RobberCar> theRobbers)
        {
            //Catch one of the Robbers
            foreach (RobberCar aRobber in theRobbers)
            {
                if (aRobber.Visible == true)
                {
                    if (aRobber.Speed == .4)
                    {
                        aRobber.Speed -= .3;
                    }
                }
            }
            
            mRobberPowerUp = new TimeSpan(0, 0, 3);
            mPowerUpText = "Easy Pickings!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void IncreaseTime(ref TimeSpan theGameTimeRemaining)
        {
            theGameTimeRemaining += new TimeSpan(0, 0, 30);

            mPowerUpText = "Time Bonus!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void DecreaseTime(ref TimeSpan theGameTimeRemaining)
        {
            theGameTimeRemaining -= new TimeSpan(0, 0, 10);

            mPowerUpText = "Time Waster!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void PrisonBust(List<RobberCar> theRobbers, ref int theRobbersCaught)
        {
            //Catch one of the Robbers
            foreach (RobberCar aRobber in theRobbers)
            {
                if (aRobber.Visible == true)
                {
                    aRobber.Visible = false;
                    theRobbersCaught += 1;
                    break;
                }
            }

            mPowerUpText = "Book 'Em!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        private void PrisonBreak(List<RobberCar> theRobbers, ref int theRobbersCaught)
        {
            //Catch one of the Robbers
            foreach (RobberCar aRobber in theRobbers)
            {
                if (aRobber.Visible == false)
                {
                    aRobber.Visible = true;
                    theRobbersCaught -= 1;
                    break;
                }
            }

            mPowerUpText = "Jail Break!";
            mPowerUpTextDisplay = new TimeSpan(0, 0, 1);
        }

        public void Reset()
        {
            mNextPowerUpAppears = new TimeSpan(0, 0, 15);
        }
    }
}
