using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class LevelScreen : BaseGame
    {
        LevelLayout mLevelLayout = new LevelLayout();     
        
        CopCar mCopCar;
        List<RobberCar> mRobberCar = new List<RobberCar>();

        PowerUpManager mPowerUps = new PowerUpManager();

        //Texture2D mFeedbackTexture;
        RenderTarget2D mFeedbackTexture;

        int mSpacing;
        int mRobbersCaught = 0;
        TimeSpan mGameTimeRemaining;      
        
        enum GameState
        {
            Running,
            Congratulations,
            GameOver,
            Paused
        }
        GameState mCurrentState = GameState.Running;

        public LevelScreen() 
        {
            mGameTimeRemaining = new TimeSpan(0, 2, 00);

            double aScale = .5;
            mSpacing = (int)(mDisplayHeight / 3) + 15;

            mCopCar = new CopCar(aScale);
            mCopCar.PositionY = mSpacing - 50;

            RobberCar aRobberCar = null;
            for (int aCounter = 0; aCounter <= 9; aCounter++)
            {
                aRobberCar = new RobberCar(aScale);
                
                switch ((int)mRandom.Next(0, 3))
                {
                    case 0:
                        {
                            aRobberCar.PositionY = 180;
                            break;
                        }
                    case 1:
                        {
                            aRobberCar.PositionY = 530;
                            break;
                        }
                    case 2:
                        {
                            aRobberCar.PositionY = 355;
                            break;
                        }
                }

                aRobberCar.PositionX += mRandom.Next(100, 700);
                mRobberCar.Add(aRobberCar);
            }

      
            //mFeedbackTexture = new Texture2D(mGraphics,
            //    mGraphics.PresentationParameters.BackBufferWidth,
            //    mGraphics.PresentationParameters.BackBufferHeight, 1,
            //    ResourceUsage.ResolveTarget, SurfaceFormat.Color, ResourceManagementMode.Manual);


            PresentationParameters pp = mGraphics.PresentationParameters;
            mFeedbackTexture = new RenderTarget2D(mGraphics,
                                               pp.BackBufferWidth,
                                               pp.BackBufferHeight,
                                               false,
                                               pp.BackBufferFormat, DepthFormat.Depth24);

            mGraphics.Clear(Color.Transparent);
            mGraphics.SetRenderTarget(mFeedbackTexture);


            //mGraphics.Clear(Color.Black);
            //mGraphics.ResolveBackBuffer(mFeedbackTexture);
        }
        
        private void RestartLevel()
        {
            mCurrentState = GameState.Running;
            
            mGameTimeRemaining = new TimeSpan(0, 2, 00);

            mPowerUps.Reset();
            
            mCopCar.PositionY = mSpacing - 50;
            mCopCar.PositionX = -500;
            mCopCar.Rotation = Car.Rotate.None;

            foreach (RobberCar aRobberCar in mRobberCar)
            {
                switch ((int)mRandom.Next(0, 3))
                {
                    case 0:
                        {
                            aRobberCar.PositionY = 180;
                            break;
                        }
                    case 1:
                        {
                            aRobberCar.PositionY = 530;
                            break;
                        }
                    case 2:
                        {
                            aRobberCar.PositionY = 355;
                            break;
                        }
                }

                aRobberCar.PositionX += mRandom.Next(100, 700);
                aRobberCar.Visible = true;
                aRobberCar.Rotation = Car.Rotate.None;
            }


            mRobbersCaught = 0;
        }        
       
        public void Update(GameTime theGameTime)
        {
            switch (mCurrentState)
            {
                case GameState.Running:
                    {
                        //Decrease the time remaining in the game
                        mGameTimeRemaining -= theGameTime.ElapsedGameTime;
                        
                        mCopCar.Update(theGameTime);
                        if (Input.Pressed(InputManager.Input.A) == true)
                        {
                            RampMovementCheck(mCopCar, 100);
                        }

                        foreach (RobberCar aRobber in mRobberCar)
                        {
                            aRobber.Update(theGameTime);
                            RampMovementCheck(aRobber, 1);

                            if (aRobber.Visible == true)
                            {
                                if (aRobber.CollisionRectangle.Intersects(mCopCar.CollisionRectangle) == true)
                                {
                                    mRobbersCaught += 1;
                                    aRobber.Visible = false;
                                }
                            }

                            if (aRobber.Rotation == Car.Rotate.None && (aRobber.PositionX > 50 && aRobber.PositionX < mDisplayWidth - 50))
                            {
                                Floor aCurrentFloor = null;

                                //Move the car down until it hits a floor
                                do
                                {
                                    aRobber.PositionY += 1;
                                    foreach (Floor aFloor in mLevelLayout.Floors)
                                    {
                                        if (aRobber.CollisionRectangle.Intersects(aFloor.CollisionRectangle) == true)
                                        {
                                            aCurrentFloor = aFloor;
                                            break;
                                        }
                                    }

                                    if (aRobber.PositionY > mDisplayHeight * 10)
                                    {
                                        break;
                                    }
                                } while (aCurrentFloor == null);

                                if (aCurrentFloor != null)
                                {
                                    //Move the car up until it just rests on the floor
                                    Rectangle aCollision = aRobber.CollisionRectangle;
                                    do
                                    {
                                        aCollision = aRobber.CollisionRectangle;
                                        aCollision.Height -= 15;
                                        if (aCollision.Intersects(aCurrentFloor.CollisionRectangle) == true)
                                        {
                                            aRobber.PositionY -= 1;
                                        }
                                    } while (aCollision.Intersects(aCurrentFloor.CollisionRectangle) == true);
                                }
                            }
                        }

                        foreach (Floor aFloor in mLevelLayout.Floors)
                        {
                            if (mCopCar.Rotation == Car.Rotate.None)
                            {
                                Rectangle aCollision = mCopCar.CollisionRectangle;
                                do
                                {
                                    aCollision = mCopCar.CollisionRectangle;
                                    aCollision.Height -= 5;
                                    if (aCollision.Intersects(aFloor.CollisionRectangle) == true)
                                    {
                                        mCopCar.PositionY -= 1;
                                    }
                                } while (aCollision.Intersects(aFloor.CollisionRectangle));
                            }
                        }


                        mPowerUps.Update(theGameTime, mCopCar, mRobberCar, ref mGameTimeRemaining, ref mRobbersCaught);

                        CheckForGameOver();
                        CheckForLevelCompletion();
                        CheckForPause();
                       
                        break;
                    }

                case GameState.Paused:
                    {
                        CheckForPause();
                        break;
                    }

                case GameState.Congratulations:
                    {
                        CheckForRestart();
                        break;
                    }

                case GameState.GameOver:
                    {
                        CheckForRestart();
                        break;
                    }
            }            
        }

        private void CheckForRestart()
        {
            if (Input.Pressed(InputManager.Input.Start) == true)
            {
                RestartLevel();
            }
        }

        private void CheckForPause()
        {
            if (Input.Pressed(InputManager.Input.Pause) == true)
            {
                if (mCurrentState == GameState.Running)
                {
                    mCurrentState = GameState.Paused;
                }
                else if (mCurrentState == GameState.Paused) 
                {
                    mCurrentState = GameState.Running;
                }
            }
        }

        private void CheckForGameOver()
        {
            //If the game time has run out, the the game is over
            if (mGameTimeRemaining <= new TimeSpan(0, 0, 0))
            {
                mCurrentState = GameState.GameOver;
            }
        }

        private void CheckForLevelCompletion()
        {
            //If all of the Robbers have been jailed, then the 
            //level has been successfully completed
            if (mRobbersCaught == mRobberCar.Count)
            {
                mCurrentState = GameState.Congratulations;
            }
        }

        private void RampMovementCheck(Car aCar, int theChance)
        {
            if (aCar.Rotation == Car.Rotate.None)
            {
                foreach (Slope aSlope in mLevelLayout.Slopes)
                {
                    if (aSlope.SlopeSlant == Slope.Slant.Right)
                    {
                        if (aCar.CollisionRectangle.Intersects(aSlope.Ramps[0].CollisionRectangle) == true)
                        {
                            if (aCar.Type == Car.CarType.Cop)
                            {
                                Vector2 aTopPosition = new Vector2(aSlope.Ramps[0].CollisionRectangle.X + 80, aSlope.Ramps[0].CollisionRectangle.Y + 50);
                                Vector2 aBottomPosition = new Vector2(aSlope.Ramps[1].CollisionRectangle.X + 60, aSlope.Ramps[1].CollisionRectangle.Y + 10);

                                double angle = Math.Atan2(aBottomPosition.Y - aTopPosition.Y, aBottomPosition.X - aTopPosition.X) * 180 / Math.PI;

                                aCar.AngleCar(aTopPosition, aBottomPosition, Car.Rotate.Down45Degrees);                                
                            }
                            else
                            {
                                if (mRandom.Next(0, 101) <= theChance)
                                {
                                    Vector2 aTopPosition = new Vector2(aSlope.Ramps[0].CollisionRectangle.X + 60, aSlope.Ramps[0].CollisionRectangle.Y + 50);
                                    Vector2 aBottomPosition = new Vector2(aSlope.Ramps[1].CollisionRectangle.X + 50, aSlope.Ramps[1].CollisionRectangle.Y + 10);

                                    aCar.AngleCar(aTopPosition, aBottomPosition, Car.Rotate.Down45Degrees);
                                }
                            }
                        }
                    }
                    else
                    {
                        if (aCar.CollisionRectangle.Intersects(aSlope.Ramps[1].CollisionRectangle) == true)
                        {
                            if (aCar.Type == Car.CarType.Cop)
                            {
                                Vector2 aTopPosition = new Vector2(aSlope.Ramps[1].CollisionRectangle.X + 50, aSlope.Ramps[1].CollisionRectangle.Y - 60);
                                Vector2 aBottomPosition = new Vector2(aSlope.Ramps[0].CollisionRectangle.X + 250, aSlope.Ramps[0].CollisionRectangle.Y - 10);

                                double angle = Math.Atan2(aBottomPosition.Y - aTopPosition.Y, aBottomPosition.X - aTopPosition.X) * 180 / Math.PI;

                                aCar.AngleCar(aTopPosition, aBottomPosition, Car.Rotate.Up45Degrees);
                            }
                            else
                            {
                                if (mRandom.Next(0, 101) <= theChance)
                                {
                                    Vector2 aTopPosition = new Vector2(aSlope.Ramps[1].CollisionRectangle.X + 60, aSlope.Ramps[1].CollisionRectangle.Y - 50);
                                    Vector2 aBottomPosition = new Vector2(aSlope.Ramps[0].CollisionRectangle.X, aSlope.Ramps[0].CollisionRectangle.Y);

                                    aCar.AngleCar(aTopPosition, aBottomPosition, Car.Rotate.Up45Degrees);
                                }
                            }
                        }
                    }
                }
            }
        }

        public void Draw(GameTime theGameTime)
        {
            mBatch.Begin();

            mLevelLayout.Draw();
            mBatch.Draw(mFeedbackTexture, new Rectangle(0, 0, (int)(mFeedbackTexture.Width * 1), (int)(mFeedbackTexture.Height * 1)), new Color(255, 255, 255, 200));

            mPowerUps.Draw();

            foreach (RobberCar aRobber in mRobberCar)
            {
                aRobber.Draw(theGameTime);
            }

            mCopCar.Draw(theGameTime);
            
            Font.DrawText(mBatch, FontManager.Alignment.None, new Vector2(70, 25), "Level 1", FontManager.Font.Instructions);
            Font.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, new Vector2(0, 25), mRobbersCaught.ToString() + " of " + mRobberCar.Count.ToString(), FontManager.Font.Instructions);
            Font.DrawText(mBatch, FontManager.Alignment.None, new Vector2(665, 25), String.Format("{0:00}:{1:00}", mGameTimeRemaining.Minutes, mGameTimeRemaining.Seconds), FontManager.Font.Instructions);
            
            switch (mCurrentState)
            {
                case GameState.Running:
                    {
                        Font.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, new Vector2(0, mDisplayHeight - mFontManager.SelectFont(FontManager.Font.Instructions).LineSpacing), "Press 'A' to enter/exit ramps", FontManager.Font.Instructions);
                        break;
                    }

                case GameState.GameOver:
                    {
                        mPowerUps.PowerUpText = string.Empty;
                        Font.DrawText(mBatch, FontManager.Alignment.CenterScreen, new Vector2(0, 0), "Game Over.", FontManager.Font.Instructions);
                        Font.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, new Vector2(0, mDisplayHeight - mFontManager.SelectFont(FontManager.Font.Instructions).LineSpacing), "Press 'Start' to play again", FontManager.Font.Instructions);
                        
                        break;
                    }

                case GameState.Congratulations:
                    {
                        mPowerUps.PowerUpText = string.Empty;
                        Font.DrawText(mBatch, FontManager.Alignment.CenterScreen, new Vector2(0, 0), "Congratulations!", FontManager.Font.Instructions);
                        Font.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, new Vector2(0, mDisplayHeight - mFontManager.SelectFont(FontManager.Font.Instructions).LineSpacing), "Press 'Start' to play again", FontManager.Font.Instructions);
                        
                        break;
                    }

                case GameState.Paused:
                    {
                        mPowerUps.PowerUpText = string.Empty;
                        mBatch.Draw(Texture.Wall, new Rectangle(0, 0, mDisplayWidth, mDisplayHeight), new Rectangle(0, 0, Texture.Wall.Width, Texture.Wall.Height), new Color(100, 100, 100, 180));
                        Font.DrawText(mBatch, FontManager.Alignment.CenterScreen, new Vector2(0, 0), "Paused.", FontManager.Font.Instructions);
                        Font.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, new Vector2(0, mDisplayHeight - mFontManager.SelectFont(FontManager.Font.Instructions).LineSpacing), "Press 'Start' to continue", FontManager.Font.Instructions);
                        
                        break;
                    }
            }      

            mBatch.End();
        
            //mGraphics.SetRenderTarget(mFeedbackTexture);

            //////Now we redraw the scene after the ResolveBackBuffer call
            //mBatch.Begin(SpriteSortMode.Immediate, BlendState.Opaque);

            //mBatch.Draw(mFeedbackTexture, new Vector2(0, 0), Color.White);
            //mBatch.End();
        }

    }
}
