using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace CopsAndRobbers
{
    class TitleScreen : BaseGame
    {        
        const double Scale_Car = 1;
        CopCar mCopCar;
        RobberCar mRobberCar;

        Rectangle mCopsRectangle;
        Rectangle mRobbersRectangle;
               
        double mScale = .50;

        int mPositionX = 100;
        int mPositionY = 150;

        int mPositionYRobber;
        RenderTarget2D feedbackTexture;

        double mTime = .5f;       

        private enum TitleState
        {
            Falling,
            BouncingAway,
            BouncingTowards,
            Still
        }
        TitleState mCurrentState = TitleState.Falling;

        private bool mStartGame = false;
        public bool StartGame
        {
            get { return mStartGame; }
        }

        public TitleScreen()
        {
            mGraphics.PresentationParameters.BackBufferFormat = SurfaceFormat.Bgr565;
            
            mPositionY = mTextureManager.CopText.Height * -1;
            mPositionYRobber = 600 + (int)(mTextureManager.CopText.Height * mScale) - 10;

            mCopsRectangle = new Rectangle(mPositionX, mPositionY, (int)(mTextureManager.CopText.Width * mScale), (int)(mTextureManager.CopText.Height * mScale));
            mRobbersRectangle = new Rectangle(mPositionX + 120, mPositionYRobber, (int)(mTextureManager.RobberText.Width * mScale), (int)(mTextureManager.RobberText.Height * mScale));
                        
            PresentationParameters pp = mGraphics.PresentationParameters;
            feedbackTexture = new RenderTarget2D(mGraphics,
                                               pp.BackBufferWidth,
                                               pp.BackBufferHeight,
                                               false,
                                               pp.BackBufferFormat, DepthFormat.Depth24);

            mGraphics.Clear(Color.Transparent);
            //mGraphics.SetRenderTarget(feedbackTexture);

            mCopCar = new CopCar(Scale_Car);
            mCopCar.PositionY = mDisplayHeight - 65;
            mCopCar.PositionX = -500;

            mRobberCar = new RobberCar(Scale_Car);
            mRobberCar.PositionY = mDisplayHeight - 50;
            mRobberCar.PositionX = -100;            
        }

        public void Update(GameTime theGameTime)
        {
            mTime -= theGameTime.ElapsedGameTime.Milliseconds;
            switch (mCurrentState)
            {
                case TitleState.Falling:
                    {
                        if (mTime < 0)
                        {
                            mTime = 1.5f;
                            mPositionY += 3;
                            mPositionYRobber -= 4;
                        }

                        if (mPositionY > 145)
                        {
                            mCurrentState = TitleState.BouncingAway;
                        }

                        break;
                    }

                case TitleState.BouncingAway:
                    {
                        if (mTime < 0)
                        {
                            mTime = 2.0f;
                            mPositionY -= 2;
                            mPositionYRobber += 4;
                        }

                        if (mPositionY < 130)
                        {
                            mCurrentState = TitleState.BouncingTowards;
                        }

                        break;
                    }

                case TitleState.BouncingTowards:
                    {
                        if (mTime < 0)
                        {
                            mTime = 4.5f;
                            mPositionY += 1;
                            mPositionYRobber -= 2;
                        }

                        if (mPositionY > 145)
                        {
                            mCurrentState = TitleState.Still;
                        }

                        break;
                    }
            }

            mCopsRectangle = new Rectangle(mPositionX, mPositionY, (int)(mTextureManager.CopText.Width * mScale), (int)(mTextureManager.CopText.Height * mScale));
            mRobbersRectangle = new Rectangle(mPositionX + 120, mPositionYRobber, (int)(mTextureManager.RobberText.Width * mScale), (int)(mTextureManager.RobberText.Height * mScale));

            if (mCurrentState == TitleState.Still)
            {               
                mCopCar.Update(theGameTime);
                mRobberCar.Update(theGameTime); 
            }

            if (mInputManager.Pressed(InputManager.Input.Start) == true)
            {
                if (mCurrentState == TitleState.Still)
                {
                    mStartGame = true;
                }
            }
        }

        public void Draw(GameTime theGameTime)
        {
            mBatch.Begin();

            //mBatch.Draw(feedbackTexture, new Rectangle(0, 0, (int)(feedbackTexture.Width * 1), (int)(feedbackTexture.Height * 1)), new Color(255, 255, 255, 227));
            mBatch.Draw(mTextureManager.CopText, mCopsRectangle, Color.White);
            mBatch.Draw(mTextureManager.RobberText, mRobbersRectangle, Color.White);

            if (mCurrentState == TitleState.Still)
            {
                Vector2 aPosition = new Vector2(0,  mGraphics.Viewport.Height - mFontManager.SelectFont(FontManager.Font.Instructions).LineSpacing - 20);
                mFontManager.DrawText(mBatch, FontManager.Alignment.CenterHorizontally, aPosition, "Press Start to Begin", FontManager.Font.Instructions);
                mCopCar.Draw(theGameTime);
                mRobberCar.Draw(theGameTime);
            }

            mBatch.End();

            //mGraphics.SetRenderTarget(feedbackTexture);

            ////Now we redraw the scene after the ResolveBackBuffer call
            //mBatch.Begin(SpriteSortMode.Immediate, null);

            //mBatch.Draw(feedbackTexture, new Vector2(0, 0), Color.White);
            //mBatch.End();

           
        }

    }
}
